@@ -29,9 +29,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 
@@ -383,8 +380,10 @@ g_vfs_backend_ftp_finalize (GObject *object)
   if (ftp->addr)
     g_object_unref (ftp->addr);
 
-  /* has been cleared on unmount */
-  g_assert (ftp->queue == NULL);
+  /* has been cleared on unmount, however it has to be cleared when mount fails */
+  if (ftp->queue)
+    g_queue_free (ftp->queue);
+
   g_cond_clear (&ftp->cond);
   g_mutex_clear (&ftp->mutex);
 
@@ -1145,7 +1144,7 @@ do_set_attribute (GVfsBackend *backend,
           if (g_vfs_ftp_task_send (&task,
                                    0,
                                    "SITE CHMOD %04o %s",
-                                   mode & (S_IRWXU | S_IRWXG | S_IRWXO),
+                                   mode,
                                    g_vfs_ftp_file_get_ftp_path (file)))
             {
               g_vfs_ftp_dir_cache_purge_file (ftp->dir_cache, file);
