@@ -132,6 +132,27 @@ struct _MetaTree {
   MetaJournal *journal;
 };
 
+/* Unfortunately the journal entries are only aligned to 32 bit boundaries
+ * but on some 64-bit RISC architectures (e.g. Alpha) this is insufficient
+ * to guarantee correct alignment of 64-bit accesses. This is not a show
+ * stopper but does cause inefficient traps to the kernel and pollution of
+ * kernel logs.  Rather than fix the alignment we provide a helper function,
+ * dependent on features specific to gcc, to correctly access a 64-bit datum
+ * that may be misaligned.
+ *
+ * See https://bugzilla.gnome.org/show_bug.cgi?id=726456
+ */
+#if defined(__GNUC__) && (defined(__alpha__) || defined(__mips__) || defined(__sparc__))
+struct una_u64 { guint64 x __attribute__((packed)); };
+static inline guint64 ldq_u(guint64 *p)
+{
+  const struct una_u64 *ptr = (const struct una_u64 *) p;
+  return ptr->x;
+}
+#else
+#define ldq_u(x) (*(x))
+#endif
+
 static void         meta_tree_refresh_locked   (MetaTree    *tree,
 						gboolean     force_reread);
 static MetaJournal *meta_journal_open          (MetaTree    *tree,
@@ -1230,7 +1251,7 @@ meta_journal_iterate (MetaJournal *journal,
           break;
         }
 
-      mtime = GUINT64_FROM_BE (entry->mtime);
+      mtime = GUINT64_FROM_BE (ldq_u (&(entry->mtime)));
       journal_path = &entry->path[0];
 
       if (journal_entry_is_key_type (entry) &&
@@ -2214,7 +2235,7 @@ apply_journal_to_builder (MetaTree *tree,
   entry = journal->first_entry;
   while (entry < journal->last_entry)
     {
-      mtime = GUINT64_FROM_BE (entry->mtime);
+      mtime = GUINT64_FROM_BE (ldq_u (&(entry->mtime)));
       journal_path = &entry->path[0];
 
       switch (entry->entry_type)
@@ -2340,18 +2361,13 @@ meta_tree_unset (MetaTree                         *tree,
   entry = meta_journal_entry_new_unset (mtime, path, key);
 
   res = TRUE;
+ retry:
   if (!meta_journal_add_entry (tree->journal, entry))
     {
       if (meta_tree_flush_locked (tree))
-        {
-	  if (!meta_journal_add_entry (tree->journal, entry))
-	  {
-	    g_warning ("meta_tree_unset: entry is bigger then the size of journal\n");
-	    res = FALSE;
-	  }
-	}
-      else
-        res = FALSE;
+	goto retry;
+
+      res = FALSE;
     }
 
   g_string_free (entry, TRUE);
@@ -2385,18 +2401,13 @@ meta_tree_set_string (MetaTree                         *tree,
   entry = meta_journal_entry_new_set (mtime, path, key, value);
 
   res = TRUE;
+ retry:
   if (!meta_journal_add_entry (tree->journal, entry))
     {
       if (meta_tree_flush_locked (tree))
-        {
-	  if (!meta_journal_add_entry (tree->journal, entry))
-	  {
-	    g_warning ("meta_tree_set_string: entry is bigger then the size of journal\n");
-	    res = FALSE;
-	  }
-	}
-      else
-        res = FALSE;
+	goto retry;
+
+      res = FALSE;
     }
 
   g_string_free (entry, TRUE);
@@ -2430,18 +2441,13 @@ meta_tree_set_stringv (MetaTree                         *tree,
   entry = meta_journal_entry_new_setv (mtime, path, key, value);
 
   res = TRUE;
+ retry:
   if (!meta_journal_add_entry (tree->journal, entry))
     {
       if (meta_tree_flush_locked (tree))
-        {
-	  if (!meta_journal_add_entry (tree->journal, entry))
-	  {
-	    g_warning ("meta_tree_set_stringv: entry is bigger then the size of journal\n");
-	    res = FALSE;
-	  }
-	}
-      else
-        res = FALSE;
+	goto retry;
+
+      res = FALSE;
     }
 
   g_string_free (entry, TRUE);
@@ -2473,18 +2479,13 @@ meta_tree_remove (MetaTree *tree,
   entry = meta_journal_entry_new_remove (mtime, path);
 
   res = TRUE;
+ retry:
   if (!meta_journal_add_entry (tree->journal, entry))
     {
       if (meta_tree_flush_locked (tree))
-        {
-	  if (!meta_journal_add_entry (tree->journal, entry))
-	  {
-	    g_warning ("meta_tree_remove: entry is bigger then the size of journal\n");
-	    res = FALSE;
-	  }
-	}
-      else
-        res = FALSE;
+	goto retry;
+
+      res = FALSE;
     }
 
   g_string_free (entry, TRUE);
@@ -2517,18 +2518,13 @@ meta_tree_copy (MetaTree                         *tree,
   entry = meta_journal_entry_new_copy (mtime, src, dest);
 
   res = TRUE;
+ retry:
   if (!meta_journal_add_entry (tree->journal, entry))
     {
       if (meta_tree_flush_locked (tree))
-        {
-	  if (!meta_journal_add_entry (tree->journal, entry))
-	  {
-	    g_warning ("meta_tree_copy: entry is bigger then the size of journal\n");
-	    res = FALSE;
-	  }
-	}
-      else
-        res = FALSE;
+	goto retry;
+
+      res = FALSE;
     }
 
   g_string_free (entry, TRUE);
